@@ -251,6 +251,14 @@ class GDAPI(object):
                     f.write(data)
         return drive_file
 
+    def request(self, method, url):
+        """https://docs.google.com/feeds/default/private/full"""
+        status_code, resp = self._googleapi.api_request(
+            method, url)
+        self._logger.debug(status_code)
+        self._logger.debug(resp.content)
+        return resp
+
     def trash_file(self, file_id):
         """Trash a file.
 
@@ -269,7 +277,7 @@ class GDAPI(object):
                 'labels': {'trashed': True},
             }
             status_code, drive_file = self._googleapi.api_request(
-                'PATCH',
+                'UPDATE',
                 '/drive/v2/files/{0}'.format(file_id),
                 data=body)
             self._logger.debug("Trash result: {0}".format(
